@@ -1,7 +1,7 @@
 #
 #   TTR: Technical Trading Rules
 #
-#   Copyright (C) 2007-2008  Joshua M. Ulrich
+#   Copyright (C) 2007-2010  Joshua M. Ulrich
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -35,8 +35,10 @@ function(price, n=14, maType, ...) {
   price <- try.xts(price, error=as.matrix)
 
   up <- momentum(price, n=1, na.pad=TRUE)
-  dn <- ifelse(up<0, abs(up), 0)
-  up <- ifelse(up>0,     up , 0)
+  which.dn <- which(up < 0)
+  dn <- up*0
+  dn[which.dn] <- -up[which.dn]
+  up[which.dn] <- 0
 
   maArgs <- list(n=n, ...)
   # Default Welles Wilder EMA
