@@ -1,7 +1,7 @@
 #
 #   TTR: Technical Trading Rules
 #
-#   Copyright (C) 2007-2008  Joshua M. Ulrich
+#   Copyright (C) 2007-2010  Joshua M. Ulrich
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -36,7 +36,12 @@ function(OHLC, n=10, calc="close", N=260, ...) {
   # Historical Close-to-Close Volatility
   # http://www.sitmo.com/eq/172
   if( calc=="close" ) {
-    r <- ROC( OHLC[,4], 1, ... )
+    # Add univariate case from Cedrick Johnson's R-SIG-Finance post
+    if( NCOL(OHLC) == 1 ) {
+      r <- ROC(OHLC[, 1], 1, ...)
+    } else {
+      r <- ROC(OHLC[, 4], 1, ...)
+    }
     rBar <- runSum( r, n-1 ) / (n-1)
     s <- sqrt( N/(n-2) * runSum( (r-rBar)^2 , n-1 ) )
   }
